def Base58ToAddress(arg):
    """

    :param arg:
    """
    pass

def AddressToBase58(arg):
    """

    :param arg:
    """
    pass

def GetCurrentBlockHash():
    """
    return current block hash
    """
    pass


