

def GetHeight():
    """

    """
    pass


def GetHeader(height_or_hash):
    """

    :param height_or_hash:
    """
    pass


def GetBlock(height_or_hash):
    """

    :param height_or_hash:
    """
    pass


def GetTransactionByHash(hash):
    """

    :param hash:
    """
    pass


def GetContract(script_hash):
    """

    :param script_hash:
    """
    pass

def GetTransactionHeight(hash):
    """
    :para hash:   
    """
    pass
